#ifndef FrameCPP__COMMON__FR_TOC_HH
#define FrameCPP__COMMON__FR_TOC_HH

#include "framecpp/Common/FrameSpec.hh"
#include "framecpp/Common/STRING.hh"
#include "framecpp/Common/FrStatData.hh"

namespace FrameCPP
{
  namespace Common
  {
    class Description;

    class FrTOC
      : public FrameSpec::Object
    {
    public:
      static const Common::FrameSpec::Info::frame_object_types s_object_id
      = Common::FrameSpec::Info::FSI_FR_TOC;

      FrTOC( const Description* Desc );

      typedef std::vector< STRING< INT_2U > > cmn_name_container_type;
      typedef std::vector< REAL_8 > cmn_dt_container_type;
      typedef std::vector< INT_4U > cmn_GTimeS_container_type;
      typedef std::vector< INT_4U > cmn_GTimeN_container_type;

      virtual ~FrTOC( );

      virtual void IndexObject( const FrameSpec::Object* Obj,
				std::streampos Position ) = 0;

      //-----------------------------------------------------------------
      // 
      //-----------------------------------------------------------------

      virtual void FrStatDataQuery( const std::string& NamePattern,
				    const General::GPSTime& StartTime,
				    const General::GPSTime& EndTime,
				    const INT_4U Version,
				    FrStatData::Query& Result ) const = 0;

      //-----------------------------------------------------------------
      // Interface routines
      //-----------------------------------------------------------------
      virtual INT_4U nFrame( ) const = 0;

      virtual const cmn_dt_container_type& dt( ) const = 0;
      virtual const cmn_GTimeS_container_type& GTimeS( ) const = 0;
      virtual const cmn_GTimeN_container_type& GTimeN( ) const = 0;

      virtual INT_8U positionH( INT_4U FrameIndex ) const = 0;
      virtual const cmn_name_container_type& nameADC( ) const = 0;
      virtual INT_8U positionADC( INT_4U FrameIndex,
				  const std::string& Channel ) const = 0;
      virtual INT_8U positionADC( INT_4U FrameIndex,
				  INT_4U Channel ) const = 0;
      virtual const cmn_name_container_type& nameProc( ) const = 0;
      virtual INT_8U positionProc( INT_4U FrameIndex,
				   const std::string& Channel ) const = 0;
      virtual INT_8U positionProc( INT_4U FrameIndex,
				   INT_4U Channel ) const = 0;
      virtual const cmn_name_container_type& nameSim( ) const = 0;
      virtual INT_8U positionSim( INT_4U FrameIndex,
				  const std::string& Channel ) const = 0;
      virtual const cmn_name_container_type& nameSer( ) const = 0;
      virtual INT_8U positionSer( INT_4U FrameIndex,
				  const std::string& Channel ) const = 0;

      virtual INT_4U nSH( ) const = 0;
      virtual INT_2U SHid( INT_4U Offset ) const = 0;
      virtual const std::string& SHname( INT_4U Offset ) const = 0;

    protected:

    private:
    };
  } // namespace - Common
} // namespace - FrameCPP

#endif /* FrameCPP__COMMON__FR_TOC_HH */
