#include "framecpp/config.h"

#include <errno.h>

#include <cstring>

#include <stdexcept>

#include "framecpp/Common/OFrameStream.hh"

FrameCPP::Common::OFrameStream::
OFrameStream( std::ostream& Stream )
  : m_stream( Stream ),
    m_is_open( false )
{
  FrameStreamBase::SetMD5Sum( m_md5_startup );
}

void FrameCPP::Common::OFrameStream::
SetMD5Sum( )
{
  m_md5_startup.Value( );
  FrameStreamBase::SetMD5Sum( );
}

void FrameCPP::Common::OFrameStream::
SetMD5Sum( MD5Sum& MD5 )
{
  if ( m_md5_startup.Finalized( ) == false )
  {
    MD5 = m_md5_startup;
    m_md5_startup.Value( );
  }
  FrameStreamBase::SetMD5Sum( MD5 );
}

    
void FrameCPP::Common::OFrameStream::
write( const void* Buffer, INT_8U Length )
{
  std::ostream::pos_type pos = m_stream.tellp( );

  calcChecksums( Buffer, Length );
  errno = 0;
  m_stream.write( (const char*)( Buffer ), Length );
  if ( ( m_stream.tellp( ) <= pos ) && ( errno != 0 ) )
  {
    std::ostringstream	msg;
    char		buffer[ 1024];

    strerror_r( errno, buffer, sizeof( buffer ) );
    msg << "Low level write error: " << buffer;
    throw std::runtime_error( msg.str( ) );
  }
}


namespace FrameCPP
{
  namespace Common
  {

    //-----------------------------------------------------------------------

    template< class T >
    OFrameStream& OFrameStream::
    operator<<( const T& Data )
    {
      m_stream.clear( );
      calcChecksums( &Data, sizeof( T ) );
      m_stream.write( (char*)( &Data ), sizeof( T ) );
      return *this;
    }

    //-----------------------------------------------------------------------

    template< class T >
    OFrameStream& OFrameStream::
    operator<<( const std::vector< T >& Data )
    {
      m_stream.clear( );
      if ( Data.size( ) > 0 )
      {
	calcChecksums( &(Data[0]), sizeof( T ) * Data.size( ) );
	m_stream.write( (char*)( &(Data[0]) ), sizeof( T ) * Data.size( ) );
      }
      return *this;
    }

#define	INSTANTIATE( LM_TYPE ) \
template OFrameStream& OFrameStream::operator<<( const LM_TYPE & ); \
template OFrameStream& OFrameStream::operator<<( const std::vector< LM_TYPE >& )
 
    /// \cond EXCLUDE
INSTANTIATE( INT_2U );
INSTANTIATE( INT_2S );
INSTANTIATE( INT_4U );
INSTANTIATE( INT_4S );
INSTANTIATE( INT_8U );
INSTANTIATE( INT_8S );
INSTANTIATE( REAL_4 );
INSTANTIATE( REAL_8 );
    /// \endcond


#undef INSTANTIATE

  } // namespace - Common
} // namespace - FrameCPP

