#ifndef FRAMECPP__COMMON__OFRAME_STREAM_HH
#define FRAMECPP__COMMON__OFRAME_STREAM_HH

#include <iostream>
#include <vector>

#include "general/types.hh"
#include "framecpp/Common/MD5Sum.hh"
#include "framecpp/Common/FrameStreamBase.hh"

namespace FrameCPP
{
  namespace Common
  {
    class OFrameStream
      : public FrameStreamBase
    {
    public:
      OFrameStream( std::ostream& Stream );

      virtual void SetMD5Sum( );

      virtual void SetMD5Sum( MD5Sum& MD5 );

      INT_8U tellp( ) const;

      void write( const void*Buffer, INT_8U Length );

      //: Write data to the stream
      template< class T >
      OFrameStream& operator<<( const T* Data );

      //: Write a vector of data to the stream
      template< class T >
      OFrameStream& operator<<( const std::vector< T >& Data );

      //: Write data to the stream
      template< class T >
      OFrameStream& operator<<( const T& Data );

    protected:
      bool is_open( ) const;
      void is_open( bool Value );
      bool good( ) const;
      void seekp( INT_8U Position, std::ios_base::seekdir Direction );

    private:
      // Output stream.
      std::ostream&	m_stream;

      bool		m_is_open;

      MD5Sum		m_md5_startup;
    };

    inline bool OFrameStream::
    good( ) const
    {
      return m_stream.good( );
    }

    inline bool OFrameStream::
    is_open( ) const
    {
      return m_is_open;
    }

    inline void OFrameStream::
    is_open( bool Value )
    {
      m_is_open = Value ;
    }

    inline void OFrameStream::
    seekp( INT_8U Position, std::ios_base::seekdir Direction )
    {
      m_stream.seekp( Position, Direction );
    }

    inline INT_8U OFrameStream::
    tellp( ) const
    {
      return m_stream.tellp( );
    }
  } // namespace - Common
} // namespace - FrameCPP

#endif /* FRAMECPP__COMMON__OFRAME_STREAM_HH */
