///=======================================================================
/// This program is useful generating a compressed frame
///
/// \todo
///     Support argument to specify the version of the frame
///     specification of the output frame file. 
//=======================================================================
#if HAVE_CONFIG_H
#include "framecpp/config.h"
#endif /* HAVE_CONFIG_H */

extern "C" {
#include <unistd.h>
} // extern "C"

#include <fstream>
#include <memory>
#include <sstream>
#include <string>

#include "general/AtExit.hh"

#include "framecpp/Common/Compression.hh"
#include "framecpp/Common/FrameStream.hh"

using std::string;
using std::filebuf;

using namespace FrameCPP::Common;

char*	Program;

string	Compression( "zero_suppress_otherwize_gzip" );
INT_4U  Level( 1 );
string	OutputName( "CompressedOut.gwf" );


inline void
depart( int Status )
{
  General::AtExit::Cleanup( );
  exit( Status );
}

static void usage( )
{
  std::cerr << "Usage: "
	    << Program << " frame_file [ frame_file ... ]" << std::endl;
}

int
main( int ArgC, char**ArgV )
try
{
  Program = ArgV[ 0 ];

  int opt;

  while( ( opt = getopt( ArgC, ArgV, "c:l:o:" ) ) != -1 )
  {
    switch( opt )
    {
    case 'c': /* compression mode */
      Compression = optarg;
      break;
    case 'l': /* compression level */
      {
	std::stringstream input( optarg );
	input >> Level;
      }
      break;
    case 'o': /* output */
      OutputName = optarg;
      break;
    default:
      break;
    } // switch( opt )
  } // while ( opt = getopt( ... )
  ArgC -= optind;
  ArgV += optind;

  if ( ArgC != 1 )
  {
    usage( );
    depart( 1 );
  } // if ( ArgC != 1 )

  //---------------------------------------------------------------------
  // Lookup the compression
  //---------------------------------------------------------------------
  INT_2U c = FrameCPP::Compression::MODE_RAW;

  if ( Compression.compare( "raw" ) == 0 )
  {
    c = FrameCPP::Compression::MODE_RAW;
  }
  else if ( Compression.compare( "zero_suppress_otherwize_gzip" ) == 0 )
  {
    c = FrameCPP::Compression::MODE_ZERO_SUPPRESS_OTHERWISE_GZIP;
  }

  //---------------------------------------------------------------------
  // Open the input stream
  //---------------------------------------------------------------------
  FrameBuffer< filebuf >*
    ibuf( new FrameBuffer< filebuf >( std::ios::in ) );
  FrameBuffer< filebuf >*
    obuf( new FrameBuffer< filebuf >( std::ios::out ) );

  ibuf->open( ArgV[ 0 ], std::ios::in | std::ios::binary );
  obuf->open( OutputName.c_str( ), std::ios::out | std::ios::binary );

  IFrameStream  iframe_stream( ibuf );
  OFrameStream  oframe_stream( obuf );

  std::auto_ptr< FrameSpec::Object >
    frame( iframe_stream.ReadFrame( true /* decompress */ ) );

  while ( frame.get( ) )
  {
    //-------------------------------------------------------------------
    // Write the source frame to the output frame file
    //-------------------------------------------------------------------
    oframe_stream.WriteFrame( *frame, c, Level );
    //-------------------------------------------------------------------
    // Read the next input frame
    //-------------------------------------------------------------------
    frame = iframe_stream.ReadFrame( );
  }
  //---------------------------------------------------------------------
  // Prepare to leave
  //---------------------------------------------------------------------
  oframe_stream.Close( );
  depart( 0 );
}
catch( std::exception& e )
{
  std::cerr << "ABORT: Caught exception: " << e.what( ) << std::endl;
  depart( 1 );
}
catch( ... )
{
  std::cerr << "ABORT: Caught unknown exception: " << std::endl;
  depart( 1 );
}
