#include "general/config.h"

#include <iostream>
#include <cstring>   

#include "regexmatch.hh"


//-----------------------------------------------------------------------------
/// \brief  Constructor
///
/// \param nmatches
///
RegexMatch::RegexMatch( size_t nmatches ) throw(std::bad_alloc)
        : mMatches( 0 ), mNMatches( nmatches ), mString( 0 )
{
    if ( mNMatches > 0 )
    {
        mMatches = new regmatch_t[ mNMatches ];
    }
}


//-----------------------------------------------------------------------------
/// \brief  Destructor
///
RegexMatch::~RegexMatch() throw()
{
    delete[] mMatches;
    mMatches = 0;
}


//-----------------------------------------------------------------------------
/// \brief  Perform a Regex Match
///
/// \param r Precompiled pattern buffer
/// \param string Text string containing the input text
/// \param eflags flag which cause changes in matching behavior
///+	(see: man regexec).
///
/// \return bool true if match was found, false otherwise
///
bool RegexMatch::match( const Regex& r, const char* string, int eflags )
    throw()
{
   mString = string;

   return (
        regexec( &r.getPattern(), mString, mNMatches, mMatches, eflags ) == 0 );
}


//-----------------------------------------------------------------------------
/// \brief  Get Substring
///
/// Return a substring as a C++ string object.
///
/// \param m Index of match starting from zero.
///
/// \return std::string - Value of the match if m is within the number of
///+	 matches, the empty string otherwise.
///
std::string RegexMatch::getSubString( size_t m ) throw()
{
    if ( getSubStart( m ) == 0 )
    {
        return "";
    }

    return std::string( getSubStart( m ), getSubLength( m ) );
}

    
